﻿#region Header
// ---------------------
// Silverlight 3 Samples
// ---------------------
//
// Project:         KeyLogger - how to detect multiple key down.
// Current Version: 1.0 / Sept 2009
//
// Copyright:       (c) 2009 Olivier Dahan
//                  odahan@e-naxos.com
//                  DotBlog: www.e-naxos.com/blog
// 
// Revision History:
// -----------------
// Sept 2009 - V 1.0 
#endregion

#region Disclaimer
// I'm the author of this source code. 
// I've placed this source code in the public domain. 
// You can use, modify, and distribute the source code and executable programs based on the source code. 
// However, note the following: 
// DISCLAIMER OF WARRANTY 
// This source code is provided "as is" and without warranties as to performance or merchantability. 
// The author and/or distributors of this source code may have made statements about this source code. 
// Any such statements do not constitute warranties and shall not be relied on by the user in deciding 
// whether to use this source code. 
// This source code is provided without any express or implied warranties whatsoever. 
// Because of the diversity of conditions and hardware under which this source code may be 
// used, no warranty of fitness for a particular purpose is offered. The user is advised 
// to test the source code thoroughly before relying on it. The user must assume the 
// entire risk of using the source code.  
#endregion

#region Usings
using System;
using System.Windows;
using System.Windows.Input;
#endregion

namespace KeyTrapper
{
    /// <summary>
    /// A class allowing to detect multiple key down.
    /// Useful for game.
    /// </summary>
    public class KeyLogger
    {
        #region Private Fields
        readonly bool[] isPressed = new bool[255];
        private FrameworkElement targetElement; 
        #endregion

        #region Property
        /// <summary>
        /// Gets or sets the target element.
        /// (Setting the target is equivalent to calling Attach);
        /// </summary>
        /// <value>The target element.</value>
        FrameworkElement TargetElement
        {
            get { return targetElement; }
            set { Attach(value); }
        } 
        #endregion

        #region Public methods
        /// <summary>
        /// Clears the key array.
        /// </summary>
        public void ClearKeyPresses()
        {
            for (int i = 0; i < 255; i++)
            {
                isPressed[i] = false;
            }
        }

        /// <summary>
        /// Clears a given key
        /// </summary>
        /// <param name="key">The k.</param>
        public void ClearKey(Key key)
        {
            isPressed[(int)key] = false;
        }

        /// <summary>
        /// Attaches the logger to the specified target.
        /// </summary>
        /// <param name="target">The target.</param>
        public void Attach(FrameworkElement target)
        {
            if (target == null)
            {
                Detach();
                return;
            }
            if (targetElement != null) Detach();
            ClearKeyPresses();
            targetElement = target;
            target.KeyDown += target_KeyDown;
            target.KeyUp += target_KeyUp;
            target.LostFocus += target_LostFocus;
        }

        /// <summary>
        /// Detaches the logger from the specified target.
        /// </summary>
        public void Detach()
        {
            if (targetElement == null) return;
            targetElement.KeyDown -= target_KeyDown;
            targetElement.KeyUp -= target_KeyUp;
            targetElement.LostFocus -= target_LostFocus;
            ClearKeyPresses();
            targetElement = null;
        }

        /// <summary>
        /// Determines whether a given key is pressed.
        /// </summary>
        /// <param name="key">The key.</param>
        /// <returns>
        /// 	<c>true</c> if the key is pressed]; otherwise, <c>false</c>.
        /// </returns>
        public bool IsKeyPressed(Key key)
        {
            return isPressed[(int)key];
        }

        /// <summary>
        /// Determines whether a given set of keys are pressed altogether.
        /// </summary>
        /// <param name="keys">The key array.</param>
        /// <returns><c>true</c> if all keys are pressed; otherwise, <c>false</c>.</returns>
        public bool AreAllKeyPressed(Key[] keys)
        {
            var i = 0;
            foreach (var k in keys)
            {
                if (IsKeyPressed(k)) i++;
            }
            return i == keys.Length;
        }
        #endregion

        #region private stuff
        private void target_KeyDown(object sender, KeyEventArgs e)
        {
            isPressed[(int)e.Key] = true;
        }

        private void target_KeyUp(object sender, KeyEventArgs e)
        {
            isPressed[(int)e.Key] = false;
        }

        private void target_LostFocus(object sender, EventArgs e)
        {
            ClearKeyPresses();
        }
        #endregion
        
    }
}